<?php

namespace App\Http\Services;

use Illuminate\Support\Facades\Http;

class OrionService
{
    public string $types;
    public string $title;
    public string $ids;
    public string $imdbid;
    public string $tmdbid;
    public string $tvdbid;
    public string $type; // movie or show
    public string|null $queryString; // blank "". null or provided string
    public string $category; // list or provided string
    protected string $searchWithoutTokens;
    protected string $searchNoTokensFiltered;
    protected string $searchWithTokens;

    public function __construct()
    {
        $this->searchNoTokensFiltered = config('orion.base_url') . config('orion.search_no_tokens_filtered');
        $this->searchWithoutTokens = config('orion.base_url') . config('orion.search_no_tokens');
        $this->searchWithTokens = config('orion.base_url') . config('orion.search_using_tokens');
    }

    /**
     * @throws \JsonException
     */
    public function genericSearch(): object|array
    {
        $url = $this->searchNoTokensFiltered . '&type='. $this->type . '&category='. $this->category;

        if (!is_null($this->queryString)) {
            $url .= '&query=' . $this->queryString;
        }
        $response = Http::get($url);
        $response = json_decode($response->body(), false, 512, JSON_THROW_ON_ERROR);
        ray($response);
        return $this->handleResponse($response);
    }

    /**
     * @throws \JsonException
     */
    public function specificSearch($title): object|array
    {
        $url = $this->searchWithoutTokens . '&type=' . $this->types .'&query='. $title;
        $response = Http::get($url);
        $response = json_decode($response->body(), false, 512, JSON_THROW_ON_ERROR);

        return $this->handleResponse($response);
    }

    public function getLinks() {
        $url = $this->searchWithTokens . '&debridlookup=realdebrid&imdb=' . $this->imdbid . '&tmdb=' . $this->tmdbid .
                '&query=' . $this->queryString . '&type=' . $this->type;
        $response = Http::get($url);
        $response = json_decode($response->body(), false, 512, JSON_THROW_ON_ERROR);
        return $this->handleResponse($response);
    }

    public function compileTypes($request): void
    {
        $types = null;

        if ($request->has('movies')) {
            $types = "movie";
        }

        if ($request->has('shows')) {
            $types .= is_null($types) ? 'show' : ',show';
        }

        if ($request->has('episodes')) {
            $types .= is_null($types) ? 'episode' : ',episode';
        }

        ray($types);

        $this->types = $types;
    }

    private function handleResponse($response): object|array
    {
        // Decode Response
        if ($response->result->status === "success") {
            return (object) [
                'data' => $response->data,
                'status' => $response->result->status
            ];
        }

        return (object) [
            "error" => "Unable to get search results at this time.",
            "status" => $response->result->status
        ];
    }
}